#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-
"""
    GESTIKK
    Version 0.2b

    Gestikk, a mouse gesture recognition application
    Copyright (C) 2007 Friedrich 'fred.reichbier' Weber (2007-10-08)

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""

from __future__ import division
import os.path
import wx
import time
import subprocess
import sys
from thread import start_new_thread
import ConfigParser

if not os.path.isdir(os.path.expanduser('~/.gestikk')): # Create directory if not existent
    os.mkdir(os.path.expanduser('~/.gestikk'))
CONFIG_FILE = os.path.expanduser('~/.gestikk/gestures.conf') # todo
wx.SetDefaultPyEncoding("iso-8859-1")

def calc_direction(pos1, pos2, shw=100):
    """ Calculate direction from pos1 to pos2, return tuple (x,y), e.g. (1,1
        Shw is the 'threshold value', as higher as more tolerant. recommended: ~ 100 """
    move_x, move_y = 0,0
    diff = (pos2[0] - pos1[0], pos2[1] - pos1[1])
    if abs(diff[0]) >= shw:
        # moved --
        if diff[0] > 0:
            move_x = 1
        elif diff[0] < 0:
            move_x = -1
        else:
            move_x = 0
    if abs(diff[1]) >= shw:
        # moved |
        if diff[1] > 0:
            move_y = 1
        elif diff[1] < 0:
            move_y = -1
        else:
            move_y = 0
    return (move_x, move_y)

def remove_duplicates(l):
    """ Remove upfollowing duplicates from the list """
    new= l[:]
    last_e = None
    for e in l:
        if e == last_e:
            new.remove(e)
        last_e = e
    return new

def my_round(i, shw=25):
    """ Not really round but use divmod.  threshold value = 5*Value """
    pow10 = 5*shw
    return (i // pow10) * pow10


class Action():
    """ Action to execute; Prototyp-Class """
    def __init__(self, *args, **kwargs):
        pass

    def do(self):
        pass

class Action_Command(Action):
    """ Aktion: Execute Command """
    def __init__(self, cmd):
        """ cmd: Command as a string """
        self.cmd = cmd

    def do(self):
        """ tu es """
        action_split = self.cmd.split(' ')
        subprocess.Popen(action_split)

class Action_Key(Action):
    """ Aktion: Press key """
    def __init__(self, key):
        """ key: Key as X-Keysym, eg. 'alt+Left' """
        self.key = key

    def do(self):
        """ tu es """
        subprocess.Popen(['xdotool', 'key', self.key], stdout=sys.stdout)

class Logic():
    """ Class for saving the logic (of gestures and preferences) """
    def __init__(self):
        self.actions = {}
        self.config = ConfigParser.ConfigParser()
        self.config_file = CONFIG_FILE # todo
        self.config.read(self.config_file)

        self.actions = self.config_get_gestures()

    def config_get_option(self, section, option, default=''):
        """ Hole Option """
        if self.config.has_section(section) and self.config.has_option(section, option):
            return self.config.get(section, option)
        else:
            return default

    def config_get_gestures(self):
        """ Fetch all gestures; Dictionary {Gesture:Action, ...} """
        if not self.config.has_section('Gestures'):
            return {}
        dic = {}
        for item in self.config.items('Gestures'): # [(1,2), (3,4)] zu {1:2, 3:4}
            # Pick an action
            # Saved in value as followed:
            # I!parameter
            # I is type of action, eg.: cmd
            art = item[1].split('!')[0]
            parameter = '!'.join(item[1].split('!')[1:])
            aktion = None
            if art == 'cmd': # Command to be executed (eg. gksu scite)
                aktion = Action_Command(parameter)
            elif art == 'key': # A Keypress, that is executed with xdotool
                aktion = Action_Key(parameter)
            dic[item[0]] = aktion
        return dic

class MGFrame(wx.Frame):
    def __init__(self, logic, parent = None, id = -1, title = "Gestikk"):
        wx.Frame.__init__(self, parent, id, title)
        self.running = True
        start_new_thread(self.mouse_gesture_thread, ())
        self.gesture_positions = [] # (x,y), (x,y), ...
        self.logic = logic

    def check_mouse_position(self):
        """ Check position of mouse pointer """
        pos = wx.GetMousePosition()
        mstate = wx.GetMouseState().RightDown()
        if mstate: # Gesture because right button is pressed
            #diff_x, diff_y = pos[0]-self.gesture[-1][0], pos[1]-self.gesture[-1][1]
            self.gesture_positions.append(pos)
        else: # button no longer pressed
            if self.gesture_positions: # end gesture
                # calculate path
                gesture = []
                tolerance = int(self.logic.config_get_option('Preferences', 'tolerance', '25'))
                for gid in range(1, len(self.gesture_positions)): # Let first out
                    gesture.append(calc_direction(self.gesture_positions[gid-1], \
                                                self.gesture_positions[gid], tolerance))
                # Remove zeros
                while (0,0) in gesture:
                    gesture.remove((0,0))
                # Delete duplicates
                gesture = remove_duplicates(gesture)
                self.gesture_positions = []
                # Is a command assigned to the gesture?
                gesture_repr = repr(gesture).replace(' ', '')[1:-1] # withour spaces and brackets
                print 'Gesture: %s' % gesture_repr
                if self.logic.actions.has_key(gesture_repr):
                    print 'Starting application with gesture '+gesture_repr
                    #self.logic.exec_action(self.logic.actions[gesture_repr])
                    self.logic.actions[gesture_repr].do() # very easy :-)


    def mouse_gesture_thread(self):
        """ Thread for gesture recognition """
        while self.running:
            wx.CallAfter(wx.YieldIfNeeded)
            wx.CallAfter(self.check_mouse_position)
            time.sleep(0.25)

def main():
    app = wx.PySimpleApp()
    logic = Logic()
    frame = MGFrame(logic)
    app.MainLoop()


if __name__ == "__main__":
    main()

